@@ -312,6 +312,42 @@ def get_most_updated_state_assessors_data(muni, path=None, include_row=False):
         return parcel_layer
     
 
+def get_most_updated_structures_data(muni):
+    """
+    pulls most updated structures data from MassGIS for a given muni
+    """
+
+    # get shapefile from a massgis link
+    shapefile_excel = (
+        "https://www.mass.gov/doc/massgis-structures-download-links/download"
+    )
+    shapefile_lookup = pd.read_excel(shapefile_excel)
+
+    town_shp_lookup_link = shapefile_lookup.loc[
+        shapefile_lookup["Municipality"] == muni.upper()
+    ]
+
+    # extract into a temporary folder for use
+
+    # path = os.path.join(project_dir, 'Data', muni) #make a subdirectory in ortho folder w town name
+    # os.makedirs(path, exist_ok=True)
+    path = tempfile.mkdtemp()
+
+    for url in town_shp_lookup_link["Structures Shapefile Link"].tolist():
+        with urlopen(url) as zipresp:
+            with zipfile.ZipFile(BytesIO(zipresp.read())) as zfile:
+                zfile.extractall(path)
+
+    layer = get_file(dir_name=path, fileType=".shp")
+    structures_layer = gpd.read_file(layer)
+ 
+    # delete temporary directory
+    shutil.rmtree(path)
+
+    structures_layer = structures_layer.to_crs(mass_mainland_crs)
+
+    return structures_layer
+
 
 def get_landuse_data(muni):
     """
@@ -1124,14 +1160,15 @@ def points_within_isochrone(
     layers_sjoin = gpd.sjoin(layer_2_in_isochrone, isochrone, predicate="within")
 
     # add a field to get count
-    layers_sjoin[new_field_name + "_count"] = 1
-
+    count_field_name = new_field_name + "_count"
+    layers_sjoin[count_field_name] = 1
+    
     ##CREATE DICTIONARY
     dict = {}
     dict[id_layer_2] = list
-    dict[new_field_name + "_count"] = "sum"
+    dict[count_field_name] = "sum"
 
-    fields_to_enrich = [id_layer_2, (new_field_name + "_count")]
+    fields_to_enrich = [id_layer_2, count_field_name]
 
     if fields:
         for field in fields:
@@ -1140,11 +1177,12 @@ def points_within_isochrone(
 
     #group by ID but aggregate with lists of values in additional fields
     isochrone_grouped = layers_sjoin.groupby(by=id_layer_1).agg(dict).reset_index()
+    
 
     #for list fields, remove quotes and brackets, only keep unique items
-    fields_to_clean = fields_to_enrich.remove((new_field_name + "_count"))
+    fields_to_enrich.remove(count_field_name)
 
-    for field in fields_to_clean:
+    for field in fields_to_enrich:
         isochrone_grouped[field] = isochrone_grouped[field].apply(lambda x: pd.Series(x).dropna().unique().tolist()).astype(
             str).str.replace('[', '').str.replace(']', '').str.replace("'", '').str.lstrip(', ').str.rstrip(',').str.rstrip(', ')
 
@@ -1153,7 +1191,7 @@ def points_within_isochrone(
         isochrone_grouped, on=id_layer_1, how="left"
     ).fillna(0)
 
-    layer_1_enriched = layer_1_enriched[layer_1_fields + fields_to_enrich]
+    layer_1_enriched = layer_1_enriched[layer_1_fields + [count_field_name] + fields_to_enrich]
 
     return layer_1_enriched
 
@@ -1211,7 +1249,8 @@ def points_within_buffer(
     )
 
     # add a field to get count
-    layers_sjoin[new_field_name + "_count"] = 1
+    count_field_name = new_field_name + "_count"
+    layers_sjoin[count_field_name] = 1
 
     # add a field for distance
     def get_distance(row):
@@ -1229,9 +1268,9 @@ def points_within_buffer(
     dict = {}
     dict[id_layer_2] = list
     dict["distance"] = list
-    dict[new_field_name + "_count"] = "sum"
+    dict[count_field_name] = "sum"
 
-    fields_to_enrich = [id_layer_2, "distance", (new_field_name + "_count")]
+    fields_to_enrich = [id_layer_2, "distance", count_field_name]
 
     if fields:
         for field in fields:
@@ -1242,18 +1281,16 @@ def points_within_buffer(
     layer_1_grouped = layers_sjoin.groupby(id_layer_1, as_index=False).agg(dict)
 
     #for list fields, remove quotes and brackets, only keep unique items
-    fields_to_clean = fields_to_enrich.remove((new_field_name + "_count"))
+    fields_to_enrich.remove(count_field_name)
 
-    for field in fields_to_clean:
+    for field in fields_to_enrich:
         layer_1_grouped[field] = layer_1_grouped[field].apply(lambda x: pd.Series(x).dropna().unique().tolist()).astype(
             str).str.replace('[', '').str.replace(']', '').str.replace("'", '').str.lstrip(', ').str.rstrip(',').str.rstrip(', ')
 
     # join back to layer_1
-    layer_1_enriched = layer_1.merge(layer_1_grouped, on=id_layer_1, how="left").fillna(
-        0
-    )
+    layer_1_enriched = layer_1.merge(layer_1_grouped, on=id_layer_1, how="left").fillna(0)
 
-    layer_1_enriched = layer_1_enriched[layer_1_fields + fields_to_enrich]
+    layer_1_enriched = layer_1_enriched[layer_1_fields + [count_field_name] + fields_to_enrich]
 
     return layer_1_enriched
 
@@ -1416,7 +1453,7 @@ def run_zonal_stats(raster_fp, zone, stat='median', nodata=None, field_name=str)
         # Get the affine
         affine = raster.transform
 
-        #calculate zonal stats (mean only) and convert into a data frame
+        #calculate zonal stats (using statitsic) and convert into a data frame
         stats = zonal_stats(zone, array, affine=affine, stats=stat, nodata=nodata, all_touched=True)
         stats_df = pd.DataFrame(stats)
 
@@ -1463,4 +1500,5 @@ def clip_raster(raster_fp, boundary):
     with rasterio.open(destination_fp,'w',**out_meta) as dst:
         dst.write(out_image)
 
+
     return destination_fp
\ No newline at end of file
